import logging
import re
from datetime import datetime, timedelta
from pyspedas.tplot_tools import time_clip as tclip
from pyspedas.tplot_tools import store_data, options
from pyspedas import dailynames, download
from .kyoto_config import CONFIG
from pyspedas import time_string,get_data, time_double, time_string, del_data, tnames
from pyspedas import tplot_copy
import numpy as np
import os

def ae_file_sort_keys(downloaded_files):
    keys = []
    for fn in downloaded_files:
        bn = os.path.basename(fn)  # like aeyymmdd
        yy = bn[2:4]
        mm = bn[4:6]
        dd = bn[6:8]
        if int(yy) >= 69:
            key = '19' + yy + mm + dd
        else:
            key = '20' + yy + mm + dd
        keys.append(key)
    return keys


def parse_ae_html(html_text):
    """
    Parses the HTML text and extracts date-times and values.

    Parameters
    ----------
    html_text : str
        The HTML text to be parsed.

    Returns
    -------
    tuple
        A tuple containing two lists - date_times and values.
        date_times : list
            A list of datetime objects representing the date and time.
        values : list
            A list of integers representing the AE values.
    """
    date_times = []
    values = []
    html_lines = html_text.split("\n")

    for line in html_lines:
        # AEALAOAU    100101L00AL PRVAE/E02
        match = re.match(
            r"A\w{7}\s+(\d{6})\w(\d{2})(AE|AL|AO|AU|AX)\s+\w{5,8}(?:/\w\d{2})?\s+(.*)",
            line)
        if match:
            date_str = match.group(1)
            hour_str = match.group(2)
            # Note: we are passing a 2-digit year to strptime!  Fortunately, the earliest date
            # for provisional AE data is 1996-01-01, and strptime treats any 2-digit year >= 69
            # as being in the 1900's, .so this is correct for our purposes.  JWL 2026-02-09
            date = datetime.strptime(date_str + hour_str, "%y%m%d%H")
            minute_values = match.group(4).split()
            for i, value in enumerate(minute_values):
                if i >= 60:
                    break  # the last values is the daily mean
                date_times.append(date + timedelta(minutes=i))
                values.append(int(value))
    return date_times, values


def load_ae_worker(
    trange=None,
    datatypes=["ae", "al", "ao", "au", "ax"],
    time_clip=True,
    remote_data_dir="",
    prefix="",
    suffix="",
    no_download=False,
    local_data_dir="",
    download_only=False,
    force_download=False,
    skip_realtime = False,
    skip_provisional = False
):
    """
    Downloads and loads either provisional or realtime index data (but not both) from the Kyoto World Data Center for Geomagnetism.

    Parameters
    ----------
    trange : list
        The time range of the data to download and load.
        Format: [start_time, end_time].
    datatypes : list, optional
        The types of AE index data to download and load.
        Default: ["ae", "al", "ao", "au", "ax"].
    time_clip : bool, optional
        Whether to clip the loaded data to the specified time range.
        Default: True.
    remote_data_dir : str, optional
        The remote directory where the data is located.
        Default: "https://wdc.kugi.kyoto-u.ac.jp/ae_provisional/".
    prefix : str, optional
        A prefix to add to the variable names of the loaded data.
        Default: "".
    suffix : str, optional
        A suffix to add to the variable names of the loaded data.
        Default: "".
    no_download : bool, optional
        If True, the data will not be downloaded, the local file will be used.
        Default: False.
    local_data_dir : str, optional
        The local directory where the data will be downloaded.
        Default: "".
    download_only : bool, optional
        If True, only download the data, do not load it.
        Default: False.
    force_download: bool
        Download file even if local version is more recent than server version
        Default: False
    skip_provisional: bool
        If Rrue, only attempt downloading and processing realtime data. Default: False
    skip_realtime: bool
        If True, only attempt downloading and processing provisional data. Default: False


    Returns
    -------
    list
        A list of tplot variable names created.

    """

    vars = []  # list of tplot variables created
    ack = """
            ******************************
            The provisional AE data are provided by the World Data Center for Geomagnetism, Kyoto,
            and are not for redistribution (https://wdc.kugi.kyoto-u.ac.jp/). Furthermore, we thank
            AE stations (Abisko [SGU, Sweden], Cape Chelyuskin [AARI, Russia], Tixi [IKFIA and
            AARI, Russia], Pebek [AARI, Russia], Barrow, College [USGS, USA], Yellowknife,
            Fort Churchill, Sanikiluaq (Poste-de-la-Baleine) [CGS, Canada], Narsarsuaq [DMI,
            Denmark], and Leirvogur [U. Iceland, Iceland]) as well as the RapidMAG team for
            their cooperations and efforts to operate these stations and to supply data for the provisional
            AE index to the WDC, Kyoto. (Pebek is a new station at geographic latitude of 70.09N
            and longitude of 170.93E, replacing the closed station Cape Wellen.)
            ******************************
        """

    if local_data_dir == "":
        local_data_dir = CONFIG["local_data_dir"]
    if local_data_dir[-1] != "/":
        local_data_dir += "/"

    if remote_data_dir == "":
        remote_data_dir = CONFIG["remote_data_dir_ae"]

    all_datatypes = ["ae", "al", "ao", "au", "ax"]
    for i in range(len(datatypes)):
        datatype = datatypes[i].lower()
        if datatype not in all_datatypes:
            logging.error("Invalid datatype: " + datatype)
            continue  # skip to the next datatype

        rt_ff = "%Y/%m/%d/" + datatype + "%y%m%d"
        prov_ff = "%Y%m/" + datatype + "%y%m%d.for.request"

        try:
            rt_file_names = dailynames(file_format=rt_ff, trange=trange)
            prov_file_names = dailynames(file_format=prov_ff, trange=trange)
        except Exception as e:
            logging.error("Error occurred while getting file names: " + str(e))
            continue  # skip to the next datatype

        # Keep unique files names only
        rt_file_names = list(set(rt_file_names))
        prov_file_names = list(set(prov_file_names))
        # The above operation does not necessarily preserve order!  We need to sort the filenames.
        rt_file_names = sorted(rt_file_names)
        prov_file_names = sorted(prov_file_names)

        times = []
        data = []
        rt_downloaded_files = []
        prov_downloaded_files = []
        # Download the html page for each url and extract times and data
        if not skip_provisional:
            for filename in prov_file_names:
                yyyymm = ""
                if len(filename) > 6:
                    yyyymm = filename[:6]
                url = remote_data_dir + "ae_provisional/" + filename
                # Use a local path similar to IDL SPEDAS
                local_path = local_data_dir + "ae_provisional/" + datatype + "/" + yyyymm + "/"
                fname = download(url, local_path=local_path, no_download=no_download, force_download=force_download)
                if fname is not None and len(fname) >= 1:
                    prov_downloaded_files.extend(fname)

        if not skip_realtime:
            for filename in rt_file_names:
                yyyy = filename[:4]
                mm = filename[5:7]
                dd = filename[8:10]
                url = remote_data_dir + "ae_realtime/data_dir/" + filename
                # Use a local path similar to IDL SPEDAS
                local_path = local_data_dir + "ae_realtime/data_dir/" + datatype + "/" + yyyy + "/" + mm + "/" + dd + "/"

                fname = download(url, local_path=local_path, no_download=no_download, force_download=force_download)
                if fname is not None and len(fname) >= 1:
                    rt_downloaded_files.extend(fname)

        if download_only:
            continue  # skip to the next data type

        if len(rt_downloaded_files) == 0 and len(prov_downloaded_files) == 0:
            logging.warning(f"No data found in requested time range for datatype {datatype}")
            continue

        # Merge prov and realtime file lists
        # We need to use YYYYMMDD as sort/duplicate keys.  The filenames only have two digit years
        # so we need to infer (guess) the century.  Fortunately, the realtime and provisional
        # file basenames are close enough that we can use the same routine to generate sort
        # keys for both.

        rt_keys = ae_file_sort_keys(rt_downloaded_files)
        prov_keys = ae_file_sort_keys(prov_downloaded_files)

        # Check for duplicates...if found, take provisional over realtime
        rt_thinned = []
        rt_keys_thinned = []
        for k,f in zip(rt_keys, rt_downloaded_files):
            if k in prov_keys:
                logging.warning(f"Both realtime and provional {datatype} data found for date {k}, using provisional")
            else:
                rt_thinned.append(f)
                rt_keys_thinned.append(k)

        combined_keys=prov_keys + rt_keys_thinned
        combined_filenames=prov_downloaded_files + rt_thinned

        # Now sort the keys and filenames with some python zip magic
        step1=zip(combined_keys, combined_filenames)
        step2=sorted(step1)
        sorted_keys_tuple, sorted_filenames_tuple = zip(*step2)
        sorted_keys = list(sorted_keys_tuple)
        sorted_filenames = list(sorted_filenames_tuple)

        # Now iterate through the time-sorted file list, load and combine the data.

        for fname in sorted_filenames:
            try:
                with open(fname, "r") as file:
                    html_text = file.read()
                file_times, file_data = parse_ae_html(html_text)
                times.extend(file_times)
                data.extend(file_data)
            except Exception as e:
                logging.error(
                    "Error occurred while parsing " + fname + ": " + str(e)
                )
                continue  # skip to the next file

        if len(times) == 0 and not download_only:
            logging.error("No data found for " + datatype + " in the given time range.")
            continue
        if not download_only:
            # Save tplot variables
            varname = prefix + "kyoto_" + datatype + suffix
            attr = {
                "source_name": "Kyoto Provisonal",
                "instrument": datatype.upper(),
                "reference": "https://wdc.kugi.kyoto-u.ac.jp/",
                "units": "nT",
                "acknowledgement": ack,
            }
            store_data(varname, data={"x": times, "y": data}, attr_dict=attr)
            options(varname, "ytitle", "Kyoto " + datatype.upper())
            options(varname, "ysubtitle", "[nT]")

            vars.append(varname)

            if time_clip:
                tclip(varname, trange[0], trange[1], overwrite=True)

    return vars


def load_ae(
        trange=None,
        datatypes=["ae", "al", "ao", "au", "ax"],
        time_clip=True,
        remote_data_dir="",
        prefix="",
        suffix="",
        no_download=False,
        local_data_dir="",
        download_only=False,
        force_download=False,
        realtime=False,
):
    """
    Downloads and loads data from the Kyoto World Data Center for Geomagnetism.
    The realtime and provisional data sets are organized a little differently, so if necessary,
    this routine makes two calls to load_ae_worker (one for provisional, one for realtime) and
    combines the results.

    Parameters
    ----------
    trange : list
        The time range of the data to download and load.
        Format: [start_time, end_time].
    datatypes : list, optional
        The types of AE index data to download and load.
        Default: ["ae", "al", "ao", "au", "ax"].
    time_clip : bool, optional
        Whether to clip the loaded data to the specified time range.
        Default: True.
    remote_data_dir : str, optional
        The remote directory where the data is located.
        Default: "https://wdc.kugi.kyoto-u.ac.jp/ae_provisional/".
    prefix : str, optional
        A prefix to add to the variable names of the loaded data.
        Default: "".
    suffix : str, optional
        A suffix to add to the variable names of the loaded data.
        Default: "".
    no_download : bool, optional
        If True, the data will not be downloaded, the local file will be used.
        Default: False.
    local_data_dir : str, optional
        The local directory where the data will be downloaded.
        Default: "".
    download_only : bool, optional
        If True, only download the data, do not load it.
        Default: False.
    force_download: bool
        Download file even if local version is more recent than server version
        Default: False

    Returns
    -------
    list
        A list of tplot variable names created.

    Examples
    --------
    >>> from pyspedas.projects.kyoto import load_ae
    >>> trange = ['2010-01-01 12:00:00', '2010-01-01 18:00:00']
    >>> vars = load_ae(trange=trange)
    >>> print(vars)
    ['kyoto_ae', 'kyoto_al', 'kyoto_ao', 'kyoto_au', 'kyoto_ax']
    """

    vars = []  # list of tplot variables created

    if trange is None or len(trange) != 2:
        logging.error("load_ae: Keyword trange with two datetimes is required to download data.")
        return vars
    trange_dbl = time_double(trange)
    earliest_data = time_double('1996-01-01')
    if trange_dbl[0] >= trange_dbl[1]:
        logging.error(f"load_ae: Invalid time range. End time {trange[1]} must be greater than start time {trange[0]}.")
        return vars
    if trange_dbl[1] < earliest_data:
        logging.error(f"load_ae: Invalid time range. End time {trange[1]} is earlier than 1996-01-01")
        return vars

    if local_data_dir == "":
        local_data_dir = CONFIG["local_data_dir"]
    if local_data_dir[-1] != "/":
        local_data_dir += "/"

    if remote_data_dir == "":
        remote_data_dir = CONFIG["remote_data_dir_ae"]

    ack = """
            ******************************
            The provisional AE data are provided by the World Data Center for Geomagnetism, Kyoto,
            and are not for redistribution (https://wdc.kugi.kyoto-u.ac.jp/). Furthermore, we thank
            AE stations (Abisko [SGU, Sweden], Cape Chelyuskin [AARI, Russia], Tixi [IKFIA and
            AARI, Russia], Pebek [AARI, Russia], Barrow, College [USGS, USA], Yellowknife,
            Fort Churchill, Sanikiluaq (Poste-de-la-Baleine) [CGS, Canada], Narsarsuaq [DMI,
            Denmark], and Leirvogur [U. Iceland, Iceland]) as well as the RapidMAG team for
            their cooperations and efforts to operate these stations and to supply data for the provisional
            AE index to the WDC, Kyoto. (Pebek is a new station at geographic latitude of 70.09N
            and longitude of 170.93E, replacing the closed station Cape Wellen.)
            ******************************
        """
    earliest_mixed_date = "2024-05-10/00:00:00"
    latest_mixed_date = "2024-05-15/00:00:00"
    latest_provisional_only = "2021-01-01/00:00:00"
    skip_realtime = False
    skip_provisional = False
    if time_double(trange[1]) <= time_double(latest_provisional_only):
        skip_realtime = True
    elif time_double(trange[0]) >= time_double(latest_mixed_date):
        skip_provisional = True

    retvalue = load_ae_worker(trange=trange,
                              datatypes=datatypes,
                              time_clip=time_clip,
                              remote_data_dir=remote_data_dir,
                              prefix=prefix,
                              suffix=suffix,
                              no_download=no_download,
                              local_data_dir=local_data_dir,
                              force_download=force_download,
                              skip_realtime=skip_realtime,
                              skip_provisional=skip_provisional,
                              )
    # Print the acknowledgments
    logging.info(ack)
    return retvalue



import unittest
import pyspedas
from pyspedas.tplot_tools import data_exists, del_data, get_data, tplot_names, time_string, time_double
import numpy as np
from pyspedas import tplot



class LoadTestCases(unittest.TestCase):
    def test_load_dst_data(self):
        # final
        del_data('*')
        dst_vars = pyspedas.projects.kyoto.dst(trange=["2015-10-15", "2015-10-16"])
        self.assertTrue(data_exists("kyoto_dst"))
        # provisional
        del_data('*')
        dst_vars = pyspedas.projects.kyoto.dst(trange=["2019-10-15", "2019-10-16"])
        self.assertTrue(data_exists("kyoto_dst"))
        # real time
        del_data('*')
        dst_vars = pyspedas.projects.kyoto.dst(trange=["2022-07-15", "2022-07-16"])
        self.assertTrue(data_exists("kyoto_dst"))

    def test_load_ae_data(self):
        # provsional before Y2K, ensure that timestamps are correctly interpreted as in the 1900s
        # The data files only contain 2-digit years(!) and we rely on strptime to do the right thing.
        del_data('*')
        ae_vars = pyspedas.projects.kyoto.load_ae(trange=["1996-01-01", "1996-01-05"])
        self.assertTrue(data_exists("kyoto_ae"))
        ae_data = get_data('kyoto_ae')
        self.assertTrue(int(ae_data.times[0]) == int(time_double('1996-01-01')))
        # provisional
        del_data('*')
        ae_vars = pyspedas.projects.kyoto.load_ae(trange=["2019-10-15", "2019-10-16"], datatypes=["ae", "al", "ao", "au", "ax"])
        self.assertTrue(data_exists("kyoto_ae"))
        self.assertTrue(data_exists("kyoto_al"))
        self.assertTrue(data_exists("kyoto_ao"))
        self.assertTrue(data_exists("kyoto_au"))
        self.assertTrue(data_exists("kyoto_ax"))
        # realtime
        del_data('*')
        ae_vars = pyspedas.projects.kyoto.load_ae(trange=["2025-10-15", "2025-10-16"], datatypes=["ae", "al", "ao", "au", "ax"])
        self.assertTrue(data_exists("kyoto_ae"))
        self.assertTrue(data_exists("kyoto_al"))
        self.assertTrue(data_exists("kyoto_ao"))
        self.assertTrue(data_exists("kyoto_au"))
        # Apparently the realtime AX index is not available yet
        #self.assertTrue(data_exists("kyoto_ax"))
        del_data('*')
        # Check a time range that straddles the provisionsl/realtime cutoff date
        ae_vars = pyspedas.projects.kyoto.load_ae(trange=["2020-12-31", "2021-01-01:23:59:59.9"], datatypes=["ae", "al", "ao", "au", "ax"])
        # AE index, should be present for both
        start_date_dbl = time_double('2020-12-31/00:00:00')
        end_date_dbl = time_double('2021-01-01/23:59:00.0')
        prov_start_date_dbl = time_double('2021-01-01/00:00:00')
        ae_data = get_data("kyoto_ae")
        self.assertTrue(ae_data.times[0] == start_date_dbl)
        self.assertTrue(ae_data.times[-1] == end_date_dbl)
        ax_data = get_data("kyoto_ax")
        # AX index, only provisional
        self.assertTrue(ax_data.times[0] == start_date_dbl)
        self.assertFalse(ax_data.times[-1] >= prov_start_date_dbl)

    def test_kyoto_ae_y2k(self):
        # Test the sort order for data files across the Y2K boundary
        # The 2-digit year rolls over from 99 to 00, so we want
        # to be sure that the data files are loaded in the right order
        # and the timestamps are monotonic
        trange=['1999-12-30','2000-01-03']
        pyspedas.projects.kyoto.load_ae(trange=trange)
        ae_data = get_data("kyoto_ae")
        dt=ae_data.times[1:]-ae_data.times[0:-1]
        dtmin=np.min(dt)
        dtmax=np.max(dt)
        print(dtmin,dtmax)
        self.assertTrue(dtmin > 0.0)

    def test_kyoto_ae_may2024(self):
        # This tests a small range that has provisional AE data for a specific event, but only
        # realtime on each side.
        del_data('*')
        trange1=['2024-05-07', '2024-05-12']
        trange2=['2024-05-12', '2024-05-21']
        trange3=['2024-05-07', '2024-05-21']

        pyspedas.projects.kyoto.load_ae(trange=trange1,datatypes=["ae"],time_clip=True,prefix='tr1_')
        #tplot('tr1_kyoto_ae')
        pyspedas.projects.kyoto.load_ae(trange=trange2,datatypes=["ae"],time_clip=True,prefix='tr2_')
        #tplot('tr2_kyoto_ae')
        pyspedas.projects.kyoto.load_ae(trange=trange3,datatypes=["ae"],time_clip=True,prefix='tr3_')
        #tplot('tr3_kyoto_ae')
        ae1 = get_data('tr1_kyoto_ae')
        ae2 = get_data('tr2_kyoto_ae')
        ae3 = get_data('tr3_kyoto_ae')
        print(time_string(ae1.times[0]))
        print(time_string(ae1.times[-1]))
        self.assertTrue(ae1.times[0] == time_double(trange1[0]))
        self.assertTrue(ae1.times[-1] > (time_double(trange1[1]) - 5*60))
        self.assertEqual(len(ae1.times) + len(ae2.times), len(ae3.times))

    def test_load_dst_3digit(self):
        # Test a time interval with 3-digit Dst values, which can run together in the data file
        del_data('*')
        dst_vars = pyspedas.projects.kyoto.dst(trange=["2015-03-16", "2015-03-19"])
        self.assertTrue(data_exists("kyoto_dst"))
        kd = get_data("kyoto_dst")
        dst_min=np.min(kd.y)
        self.assertTrue(dst_min < -100.0)

    def test_load_geomagnetic_indices(self):
        del_data('*')
        geom_ind_vars = pyspedas.projects.kyoto.load_geomagnetic_indices(trange=["2015-03-16", "2015-03-19"])
        self.assertTrue(data_exists("kyoto_ae"))
        self.assertTrue(data_exists("thg_idx_al"))
        self.assertTrue(data_exists("noaa_Kp"))
        self.assertTrue(data_exists("gfz_Kp"))
        self.assertTrue(data_exists("omni_Pressure"))
        self.assertTrue("kyoto_ae" in geom_ind_vars)
        self.assertTrue("thg_idx_al" in geom_ind_vars)
        self.assertTrue("noaa_Kp" in geom_ind_vars)
        self.assertTrue("gfz_Kp" in geom_ind_vars)
        self.assertTrue("omni_Pressure" in geom_ind_vars)

    def test_load_geomagnetic_indices_omni_load_all(self):
        del_data('*')
        geom_ind_vars = pyspedas.projects.kyoto.load_geomagnetic_indices(missions=["omni"], omni_load_all=True,trange=["2015-03-16", "2015-03-19"])
        self.assertTrue(data_exists("omni_BY_GSE"))
        self.assertTrue("omni_BY_GSE" in geom_ind_vars)

    def test_load_geomagnetic_indices_datatype(self):
        del_data('*')
        geom_ind_vars = pyspedas.projects.kyoto.load_geomagnetic_indices(trange=["2015-03-16", "2015-03-19"], datatypes=['dst'])
        self.assertTrue(data_exists("kyoto_dst"))
        self.assertTrue("kyoto_dst" in geom_ind_vars)

    def test_errors(self):
        pyspedas.projects.kyoto.dst(trange=None)
        pyspedas.projects.kyoto.dst(trange=["2020-04-01","2020-03-01"])
        pyspedas.projects.kyoto.dst(trange=["1015-10-15", "1015-10-16"])
        pyspedas.projects.kyoto.load_ae(trange=None)
        pyspedas.projects.kyoto.load_ae(trange=["2020-04-01","2020-03-01"])
        pyspedas.projects.kyoto.load_ae(trange=["1015-10-15", "1015-10-16"])


if __name__ == "__main__":
    unittest.main()



from .maven_load import load_data
import pyspedas.projects.maven.spdf as spdf_load

# This routine was originally in maven/__init__.py, until being moved to its own file.
# Please refer to __init__.py if you need to see the revision history before it was moved.

maven_load = load_data

def sep(
    trange=["2016-01-01", "2016-01-02"],
    level="l2",
    datatype="s2-cal-svy-full",
    suffix="",
    prefix="",
    varformat=None,
    get_support_data=False,
    auto_yes=True,
    downloadonly=False,
    varnames=[],
    spdf=False,
):
    """
    Loads MAVEN Solar Energetic Particle (SEP) data.

    Parameters
    ----------
    trange : list, optional
        Time range of the data in the format ["start_date", "end_date"].
        Defaults to ["2016-01-01", "2016-01-02"].
    level : str, optional
        Data level. Defaults to "l2".
    datatype : str, optional
        Data type. Defaults to "s2-cal-svy-full".
    suffix: str
        The tplot variable names will be given this suffix.
        Default: '', no suffix is added.
    prefix: str
        The tplot variable names will be given this prefix.
        Default: '', no prefix is added.
    varformat : str, optional
        Variable format. Defaults to None.
    get_support_data : bool, optional
        Whether to download support data. Defaults to False.
    auto_yes : bool, optional
        Whether to automatically answer 'yes' to all prompts. Defaults to True.
    downloadonly : bool, optional
        Whether to only download the data without loading it. Defaults to False.
    varnames : list, optional
        List of variable names to load. Defaults to [].
    spdf : bool, optional
        Whether to use the SPDF library for loading the data. Defaults to False.

    Returns
    -------
    dict
        Dictionary of loaded data variables.
    """

    if spdf:
        return spdf_load.sep(
            trange=trange,
            level=level,
            datatype=datatype,
            varformat=varformat,
            get_support_data=get_support_data,
            downloadonly=downloadonly,
            varnames=varnames,
        )
    return maven_load(
        instruments="sep",
        start_date=trange[0],
        end_date=trange[1],
        type=datatype,
        level=level,
        varformat=varformat,
        suffix=suffix,
        prefix=prefix,
        get_support_data=get_support_data,
        auto_yes=auto_yes,
        download_only=downloadonly,
        varnames=varnames,
    )




import datetime
import numpy as np
from typing import Any

def is_timezone_aware(dt:Any)->bool:
    """
    Return True if input is a timezone-aware datetime object.

    Parameters:
    ----------
    dt: Any
    An object, or list or Numpy array of objects to be checked.
    If dt is a list or array, the first element is checked.

    Returns
    -------
    bool
    True if input is a timezone-aware datetime object, false otherwise.

    """
    if isinstance(dt, list) or isinstance(dt,np.ndarray):
        obj = dt[0]
    else:
        obj = dt

    if isinstance(obj,datetime.datetime):
        return obj.tzinfo is not None and obj.tzinfo.utcoffset(obj) is not None
    else:
        return False



from pyspedas.projects.mms.mms_load_data import mms_load_data
from pyspedas.projects.mms.edi_tools.mms_edi_set_metadata import mms_edi_set_metadata
from pyspedas.projects.mms.mms_config import CONFIG



def mms_load_edi(trange=['2016-10-16', '2016-10-17'], probe='1', data_rate='srvy', level='l2', datatype='efield',
        varformat=None, varnames=[], get_support_data=False, suffix='', time_clip=False, no_update=False,
        available=False, notplot=False, latest_version=False, major_version=False, min_version=None, cdf_version=None, 
        spdf=False, always_prompt=False):
    """
    Load data from the MMS Electron Drift Instrument (EDI)
    
    Parameters
    ----------
        trange : list of str
            time range of interest [start time, end time] with the format
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']
            Default: ['2015-10-16', '2015-10-17']

        probe : str or list of str
            list of probes, valid values for MMS probes are ['1','2','3','4'].
            Default: '1'

        data_rate : str or list of str
            instrument data rates for EDI include ['brst', 'fast', 'slow', 'srvy'].
            Default: 'srvy'

        level : str
            indicates level of data processing.
            Default: 'l2'

        datatype : str or list of str
            Valid datatypes for EDI are: ['efield', 'amb']
            Default: 'efield'

        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.
            Default: False

        time_clip: bool
            Data will be clipped to the exact trange specified by the trange keyword.
            Default: False
            
        varformat: str
            The file variable formats to load into tplot.  Wildcard characters ``* and ?`` are accepted.
            Default: None (all variables are loaded)

        varnames: list of str
            List of variable names to load If list is empty or not specified,
            all data variables are loaded
            Default: []

        suffix: str
            The tplot variable names will be given this suffix.
            Default: None

        notplot: bool
            If True, then data are returned in a hash table instead of 
            being stored in tplot variables (useful for debugging, and
            access to multidimensional data products)
            Default: False

        available: bool
            If True, simply return the available data files (without downloading)
            for the requested parameters
            Default: False

        no_update: bool
            Set this flag to preserve the original data. if not set and newer 
            data is found the existing data will be overwritten
            Default: False

        cdf_version: str
            Specify a specific CDF version number to load (e.g., cdf_version='4.3.0')
            Default: None

        min_version: str
            Specify a minimum CDF version number to load
            Default: None

        latest_version: bool
            Only grab the latest CDF version in the requested time interval
            Default: False

        major_version: bool
            Only open the latest major CDF version (e.g., X in vX.Y.Z) in the requested time interval
            Default: False

        always_prompt: bool
            Set this keyword to always prompt for the user's username and password;
            useful if you accidentally save an incorrect password, or if your SDC password has changed
            Default: False

        spdf: bool
            If True, download the data from the SPDF instead of the SDC
            Default: False

    Returns
    --------
        list of str
            List of tplot variables created.

    Example
    -------

    >>> import pyspedas
    >>> from pyspedas import tplot
    >>> edi_vars = pyspedas.projects.mms.mms_load_edi(trange=['2016-10-16', '2016-10-17'], probe='1')
    >>> tplot('mms1_edi_e_gse_srvy_l2')


    """
    tvars = mms_load_data(trange=trange, notplot=notplot, probe=probe, data_rate=data_rate, level=level, instrument='edi',
            datatype=datatype, varformat=varformat, varnames=varnames, get_support_data=get_support_data, suffix=suffix, time_clip=time_clip, 
            no_update=no_update, available=available, latest_version=latest_version, major_version=major_version, 
            min_version=min_version, cdf_version=cdf_version, spdf=spdf, always_prompt=always_prompt)

    if tvars is None or available or notplot or CONFIG['download_only']:
        return tvars

    mms_edi_set_metadata(probe, data_rate, level, suffix=suffix)

    return tvars
